# RhodeCode VCSServer provides access to different vcs backends via network.
# Copyright (C) 2014-2020 RhodeCode GmbH
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import time
import logging

import vcsserver
from vcsserver.utils import safe_str, ascii_str

log = logging.getLogger(__name__)


def get_access_path(environ):
    path = environ.get('PATH_INFO')
    return path


def get_user_agent(environ):
    return environ.get('HTTP_USER_AGENT')


def get_vcs_method(environ):
    return environ.get('HTTP_X_RC_METHOD')


def get_vcs_repo(environ):
    return environ.get('HTTP_X_RC_REPO_NAME')


class RequestWrapperTween(object):
    def __init__(self, handler, registry):
        self.handler = handler
        self.registry = registry

        # one-time configuration code goes here

    def __call__(self, request):
        start = time.time()
        log.debug('Starting request time measurement')
        response = None

        ua = get_user_agent(request.environ)
        vcs_method = get_vcs_method(request.environ)
        repo_name = get_vcs_repo(request.environ)

        try:
            response = self.handler(request)
        finally:
            count = request.request_count()
            _ver_ = ascii_str(vcsserver.__version__)
            _path = safe_str(get_access_path(request.environ))
            ip = '127.0.0.1'
            match_route = request.matched_route.name if request.matched_route else "NOT_FOUND"
            resp_code = getattr(response, 'status_code', 'UNDEFINED')

            total = time.time() - start

            _view_path = f"{repo_name}@{_path}/{vcs_method}"
            log.info(
                'Req[%4s] IP: %s %s Request to %s time: %.4fs [%s], VCSServer %s',
                count, ip, request.environ.get('REQUEST_METHOD'),
                _view_path, total, ua, _ver_,
                extra={"time": total, "ver": _ver_, "code": resp_code,
                       "path": _path, "view_name": match_route, "user_agent": ua,
                       "vcs_method": vcs_method, "repo_name": repo_name}
            )

            statsd = request.registry.statsd
            if statsd:
                match_route = request.matched_route.name if request.matched_route else _path
                elapsed_time_ms = round(1000.0 * total)  # use ms only
                statsd.timing(
                    "vcsserver_req_timing.histogram", elapsed_time_ms,
                    tags=[
                        "view_name:{}".format(match_route),
                        "code:{}".format(resp_code)
                    ],
                    use_decimals=False
                )
                statsd.incr(
                    "vcsserver_req_total", tags=[
                        "view_name:{}".format(match_route),
                        "code:{}".format(resp_code)
                    ])

        return response


def includeme(config):
    config.add_tween(
        'vcsserver.tweens.request_wrapper.RequestWrapperTween',
    )
